\name{chi.squared.test}
\alias{chi.squared.test}
\title{
  Pearson's Chi-squared Test for Contingency Tables
}
\description{
  Performs performs the chi-squared contingency table test
}
\usage{
chi.squared.test(x, y = NULL, conservative = FALSE, cramers.v.conf=.95, simulate.p.value = FALSE, B = 10000) 
}
\arguments{
  \item{x}{A vector or a matrix}
  \item{y}{A vector that is ignored if x is a matrix and required if x is a vector}
  \item{conservative}{If \code{FALSE}, the usual chi-squared test is performed 
  and if \code{simulate.p.value=TRUE}, the monte carlo mid p-value is returned. 
  Otherwise, Yates' continuity correction is applied, and if \code{simulate.p.value=TRUE} 
  the monte carlo conservative p-value is returned}
  \item{cramers.v.conf}{The confidence level for the confidence interval around the Cramer's V effect size}
  \item{simulate.p.value}{a logical indicating whether to compute p-values by monte carlo simulation}
  \item{B}{An integer specifying the number of replicates used in the monte carlo test}
}
\value{
A list with class "htest" containing the following components:
\item{statistic}{
the value the chi-squared test statistic. }
\item{parameter}{
the degrees of freedom of the approximate chi-squared distribution of the test statistic.
}
\item{p.value}{the p-value for the test.}
\item{method}{a character string indicating the type of test performed, 
and whether Monte Carlo simulation or continuity correction was used.}
\item{data.name}{a character string giving the name(s) of the data.}
\item{observed}{the observed counts.}
\item{expected}{the expected counts under the null hypothesis.}
\item{residuals}{the Pearson residuals, (observed - expected) / sqrt(expected).}
\item{estimate}{Cramer's V}
\item{conf.int}{A confidence interval for Cramer's V}
}
\details{
If x is a matrix with at least two rows and columns, it is taken as a two-dimensional contingency table. 
The entries of x must be non-negative integers. Otherwise, x and y must be vectors or factors of the 
same length; incomplete cases are removed, the objects are coerced into factor objects, and the 
contingency table is computed from these. Then, Pearson's chi-squared test of the null hypothesis 
that the joint distribution of the cell counts in a 2-dimensional contingency table is the product of 
the row and column marginals is performed.

If \code{simulate.p.value} is \code{FALSE} and \code{conservative} is \code{FALSE} the p-value is 
computed from the asymptotic chi-squared distribution of the test statistic; continuity correction 
is not applied.

If \code{simulate.p.value} is \code{TRUE} and \code{conservative} is \code{FALSE} the p-value is 
computed via for a Monte Carlo test (Hope, 1968) with B replicates. The mid p-value is returned. The 
mid p-value is defined as the proportion of replicates less than the observed chi-squared value plus 
one half times the proportion of replicates equal to the observed chi-squared value.

If \code{simulate.p.value} is \code{FALSE} and \code{conservative} is \code{TRUE} the p-value is 
computed from the asymptotic chi-squared distribution of the test statistic with the Yates
continuity correction applied only in the case of a 2-by-2 table.

If \code{simulate.p.value} is \code{TRUE} and \code{conservative} is \code{TRUE} the p-value is 
computed via for a Monte Carlo test (Hope, 1968) with B replicates. The conservative p-value is returned, which
is defined as the proportion of replicates less than or equal to the observed chi-squared value.

In the contingency table case simulation is done by random sampling from the set of all 
contingency tables with given marginals, and works only if the marginals are strictly positive. 
(A C translation of the algorithm of Patefield (1981) is used.) Continuity correction is never used, 
and the statistic is quoted without it. Note that this is not the usual sampling situation for the 
chi-squared test but rather that for Fisher's exact test.
}
\references{
Hope, A. C. A. (1968) A simplified Monte Carlo significance test procedure. J. Roy, Statist. Soc. B 30.

Patefield, W. M. (1981) Algorithm AS159. An efficient method of generating r x c tables with 
			given row and column totals. Applied Statistics 30.
}
\seealso{
  \code{\link{chisq.test}}
  \code{\link{likelihood.test}}
}
\examples{
data(InsectSprays)
chi.squared.test(InsectSprays$count>7,InsectSprays$spray)
}
 