\name{fishers.exact.test}
\alias{fishers.exact.test}
\title{
  Fisher's Exact Test of independence
}
\description{
  Performs Fisher's exact test of independence in a contingency table conditional on the marginal totals
}
\usage{
fishers.exact.test(x, y = NULL, conservative=FALSE, simulate.p.value = FALSE, B = 10000,
					workspace = 2e+05, control = list()) 
}
\arguments{
  \item{x}{A vector or a matrix}
  \item{y}{A vector that is ignored if x is a matrix and required if x is a vector}
  \item{conservative}{If \code{FALSE}, the exact mid p-value value is returned; 
  Otherwise the exact conservative p-value is used.}
  \item{simulate.p.value}{a logical indicating whether to compute p-values by Monte Carlo simulation}
  \item{B}{The number of replications to use in the simulation}
  \item{workspace}{an integer specifying the size of the workspace used in the network algorithm. 
  In units of 4 bytes. Only used for non-simulated p-values larger than 2 by 2 tables.}
  \item{control}{a list with named components for low level algorithm control. At present the only one used is
  "mult", a positive integer >= 2 with default 30 used only for larger than 2 by 2 tables. This says how many 
  times as much space should be allocated to paths as to keys: see file fexact.c in the sources of 
  the package 'stats'.}
}
\value{
A list with class "htest" containing the following components:
\item{p.value}{the p-value for the test.}
\item{method}{a character string indicating the type of test performed, 
and whether Monte Carlo simulation or continuity correction was used.}
\item{data.name}{a character string giving the name(s) of the data.}
\item{statistic}{currently always \code{NA}.}
\item{parameter}{currently always \code{NA}.}
}
\details{
The same as \code{\link{fisher.test}} except that the mid p-value is supported, and is the default.
}
\seealso{
  \code{\link{fisher.test}}
  \code{\link{chi.squared.test}}
  \code{\link{likelihood.test}}
}
\examples{
## A r x c table  Agresti (2002, p. 57) Job Satisfaction
Job <- matrix(c(1,2,1,0, 3,3,6,1, 10,10,14,9, 6,7,12,11), 4, 4,
dimnames = list(income=c("< 15k", "15-25k", "25-40k", "> 40k"),
                satisfaction=c("VeryD", "LittleD", "ModerateS", "VeryS")))
fishers.exact.test(Job)

}  